import torch
from fairseq.checkpoint_utils import load_model_ensemble_and_task
from fairseq.models.hubert import HubertModel
from huggingface_hub import hf_hub_download


def load_hubert(
    hubert: str | HubertModel | None = None,
    device: torch.device = torch.get_default_device(),
) -> HubertModel:
    """
    Load the Hubert model from a file or download it if necessary.
    If a loaded model is provided, it will be returned as is.

    Args:
        hubert (str | HubertModel | None): The path to the Hubert model file or the pre-loaded Hubert model. If None, the default model will be downloaded.
        device (torch.device): The device to load the model on.

    Returns:
        HubertModel: The loaded Hubert model.

    Raises:
        If the model file does not exist.
    """
    if isinstance(hubert, HubertModel):
        return hubert.to(device)
    if isinstance(hubert, str):
        models, _, _ = load_model_ensemble_and_task([hubert])
        model = models[0].to(device)
        return model
    hubert = hf_hub_download("lj1995/VoiceConversionWebUI", "hubert_base.pt")
    models, _, _ = load_model_ensemble_and_task([hubert])
    model = models[0].to(device)
    return model
